"""
All exceptions thrown by service_identity.

Separated into an own package for nicer tracebacks, you should still import
them from __init__.py.
"""

from __future__ import absolute_import, division, print_function

from characteristic import attributes


@attributes(["errors"])
class VerificationError(Exception):
    """
    Service identity verification failed.
    """


@attributes(["mismatched_id"])
class DNSMismatch(object):
    """
    Not matching DNSPattern could be found.
    """


@attributes(["mismatched_id"])
class SRVMismatch(object):
    """
    Not matching SRVPattern could be found.
    """


@attributes(["mismatched_id"])
class URIMismatch(object):
    """
    Not matching URIPattern could be found.
    """


class CertificateError(Exception):
    """
    Certificate contains invalid or unexpected data.
    """



from __future__ import absolute_import, division, print_function

from OpenSSL.test.util import TestCase

from service_identity._common import DNSPattern, URIPattern
from service_identity.pyopenssl import extract_ids, verify_hostname
from .util import CERT_CN_ONLY, CERT_DNS_ONLY, CERT_OTHER_NAME


class VerifyHostnameTestCase(TestCase):
    def test_verify_hostname(self):
        """
        It's just a convenience one-liner.  Let's check it doesn't explode b/c
        of some typo.
        """
        class FakeConnection(object):
            def get_peer_certificate(self):
                return CERT_DNS_ONLY

        verify_hostname(FakeConnection(), u"twistedmatrix.com")


class ExtractIDsTestCase(TestCase):
    def test_dns(self):
        """
        Returns the correct DNSPattern from a certificate.
        """
        rv = extract_ids(CERT_DNS_ONLY)
        self.assertEqual(
            [DNSPattern(b'www.twistedmatrix.com'),
             DNSPattern(b'twistedmatrix.com')],
            rv
        )

    def test_cn_ids_are_used_as_fallback(self):
        """
        CNs are returned as DNSPattern if no other IDs are present.
        """
        rv = extract_ids(CERT_CN_ONLY)
        self.assertEqual(
            [DNSPattern(b'www.microsoft.com')], rv
        )

    def test_uri(self):
        """
        Returns the correct URIPattern from a certificate.
        """
        rv = extract_ids(CERT_OTHER_NAME)
        self.assertEqual(
            [URIPattern(b'http://example.com/')],
            [id for id in rv if isinstance(id, URIPattern)]
        )



import codecs
import os
import re

from setuptools import setup, find_packages


def read(*parts):
    """
    Build an absolute path from *parts* and and return the contents of the
    resulting file.  Assume UTF-8 encoding.
    """
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, *parts), 'rb', 'utf-8') as f:
        return f.read()


def find_version(*file_paths):
    """
    Build a path from *file_paths* and search for a ``__version__``
    string inside.
    """
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


if __name__ == "__main__":
    setup(
        description="Service identity verification for pyOpenSSL.",
        long_description=(
            read("README.rst") + "\n\n" +
            read("AUTHORS.rst")
        ),
        install_requires=[
            "characteristic",
            "pyasn1",
            "pyasn1-modules",
            "pyopenssl>=0.12",
        ],
        keywords="cryptography openssl pyopenssl",
        license="MIT",
        name="service_identity",
        packages=find_packages(exclude=['tests*']),
        url="https://github.com/pyca/service_identity",
        version=find_version('service_identity/__init__.py'),
        maintainer='Hynek Schlawack',
        maintainer_email='hs@ox.cx',
        classifiers=[
            "Development Status :: 5 - Production/Stable",
            "Intended Audience :: Developers",
            'License :: OSI Approved :: MIT License',
            "Natural Language :: English",
            "Operating System :: MacOS :: MacOS X",
            "Operating System :: POSIX",
            "Operating System :: POSIX :: BSD",
            "Operating System :: POSIX :: Linux",
            "Operating System :: Microsoft :: Windows",
            "Programming Language :: Python",
            "Programming Language :: Python :: 2",
            "Programming Language :: Python :: 2.6",
            "Programming Language :: Python :: 2.7",
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3.3",
            "Programming Language :: Python :: 3.4",
            "Programming Language :: Python :: Implementation :: CPython",
            "Programming Language :: Python :: Implementation :: PyPy",
            "Topic :: Security :: Cryptography",
        ],
    )



"""
Verify service identities.
"""

from __future__ import absolute_import, division, print_function

__version__ = "1.0.0"
__author__ = "Hynek Schlawack"
__license__ = "MIT"

from . import pyopenssl
from .exceptions import (
    CertificateError,
    VerificationError,
)

__all__ = [
    "CertificateError",
    "VerificationError",
    "pyopenssl",
]
